@@ -8,7 +8,13 @@ import asyncio
 import email.message
 import socket
 import ssl
-from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Type, Union
+from collections.abc import Iterable, Sequence
+from typing import (
+    Any,
+    Literal,
+    Optional,
+    Union,
+)
 
 from .auth import auth_crammd5_verify, auth_login_encode, auth_plain_encode
 from .email import (
@@ -36,7 +42,7 @@ from .errors import (
 from .esmtp import parse_esmtp_extensions
 from .protocol import SMTPProtocol
 from .response import SMTPResponse
-from .typing import Default, SMTPStatus, SocketPathType, _default
+from .typing import Default, SMTPStatus, SocketPathType
 
 
 __all__ = ("SMTP", "SMTP_PORT", "SMTP_TLS_PORT", "SMTP_STARTTLS_PORT")
@@ -53,15 +59,13 @@ class SMTP:
 
     Basic usage:
 
-        >>> event_loop = asyncio.get_event_loop()
         >>> smtp = aiosmtplib.SMTP(hostname="127.0.0.1", port=1025)
-        >>> event_loop.run_until_complete(smtp.connect())
-        (220, ...)
         >>> sender = "root@localhost"
         >>> recipients = ["somebody@localhost"]
-        >>> message = "Hello World"
-        >>> send = smtp.sendmail(sender, recipients, "Hello World")
-        >>> event_loop.run_until_complete(send)
+        >>> async def connect_and_send():
+        ...     await smtp.connect()
+        ...     return await smtp.sendmail(sender, recipients, "Hello")
+        >>> asyncio.run(connect_and_send())
         ({}, 'OK')
 
     Keyword arguments can be provided either on :meth:`__init__` or when
@@ -73,7 +77,7 @@ class SMTP:
     """
 
     # Preferred methods first
-    AUTH_METHODS: Tuple[str, ...] = (
+    AUTH_METHODS: tuple[str, ...] = (
         "cram-md5",
         "plain",
         "login",
@@ -82,12 +86,12 @@ class SMTP:
     def __init__(
         self,
         *,
-        hostname: Optional[str] = "localhost",
+        hostname: Optional[str] = None,
         port: Optional[int] = None,
         username: Optional[Union[str, bytes]] = None,
         password: Optional[Union[str, bytes]] = None,
         local_hostname: Optional[str] = None,
-        source_address: Optional[Tuple[str, int]] = None,
+        source_address: Optional[tuple[str, int]] = None,
         timeout: Optional[float] = DEFAULT_TIMEOUT,
         use_tls: bool = False,
         start_tls: Optional[bool] = None,
@@ -100,19 +104,19 @@ class SMTP:
         sock: Optional[socket.socket] = None,
     ) -> None:
         """
-        :keyword hostname:  Server name (or IP) to connect to. Defaults to "localhost".
-        :keyword port: Server port. Defaults ``465`` if ``use_tls`` is ``True``,
+        :keyword hostname:  Server name (or IP) to connect to. defaults to "localhost".
+        :keyword port: Server port. defaults ``465`` if ``use_tls`` is ``True``,
             ``587`` if ``start_tls`` is ``True``, or ``25`` otherwise.
         :keyword username:  Username to login as after connect.
         :keyword password:  Password for login after connect.
         :keyword local_hostname: The hostname of the client.  If specified, used as the
             FQDN of the local host in the HELO/EHLO command. Otherwise, the result of
-            :func:`socket.getfqdn`. **Note that getfqdn will block the event loop.**
+            :func:`socket.getfqdn`.
         :keyword source_address: Takes a 2-tuple (host, port) for the socket to bind to
             as its source address before connecting. If the host is '' and port is 0,
             the OS default behavior will be used.
         :keyword timeout: Default timeout value for the connection, in seconds.
-            Defaults to 60.
+            defaults to 60.
         :keyword use_tls: If True, make the initial connection to the server
             over TLS/SSL. Mutually exclusive with ``start_tls``; if the server uses
             STARTTLS, ``use_tls`` should be ``False``.
@@ -123,7 +127,7 @@ class SMTP:
             If ``False``, no upgrade will occur.
             Mutually exclusive with ``use_tls``.
         :keyword validate_certs: Determines if server certificates are
-            validated. Defaults to ``True``.
+            validated. defaults to ``True``.
         :keyword client_cert: Path to client side certificate, for TLS.
         :keyword client_key: Path to client side key, for TLS.
         :keyword tls_context: An existing :py:class:`ssl.SSLContext`, for TLS.
@@ -144,7 +148,7 @@ class SMTP:
         self.port = port
         self._login_username = username
         self._login_password = password
-        self._local_hostname = local_hostname
+        self.local_hostname = local_hostname
         self.timeout = timeout
         self.use_tls = use_tls
         self._start_tls_on_connect = start_tls
@@ -161,9 +165,9 @@ class SMTP:
         self._connect_lock: Optional[asyncio.Lock] = None
         self.last_helo_response: Optional[SMTPResponse] = None
         self._last_ehlo_response: Optional[SMTPResponse] = None
-        self.esmtp_extensions: Dict[str, str] = {}
+        self.esmtp_extensions: dict[str, str] = {}
         self.supports_esmtp = False
-        self.server_auth_methods: List[str] = []
+        self.server_auth_methods: list[str] = []
         self._sendmail_lock: Optional[asyncio.Lock] = None
 
         self._validate_config()
@@ -175,7 +179,7 @@ class SMTP:
         return self
 
     async def __aexit__(
-        self, exc_type: Type[BaseException], exc: BaseException, traceback: Any
+        self, exc_type: type[BaseException], exc: BaseException, traceback: Any
     ) -> None:
         if isinstance(exc, (ConnectionError, TimeoutError)):
             self.close()
@@ -193,17 +197,6 @@ class SMTP:
         """
         return bool(self.protocol is not None and self.protocol.is_connected)
 
-    @property
-    def local_hostname(self) -> str:
-        """
-        Get the system hostname to be sent to the SMTP server.
-        Simply caches the result of :func:`socket.getfqdn`.
-        """
-        if self._local_hostname is None:
-            self._local_hostname = socket.getfqdn()
-
-        return self._local_hostname
-
     @property
     def last_ehlo_response(self) -> Union[SMTPResponse, None]:
         return self._last_ehlo_response
@@ -228,7 +221,7 @@ class SMTP:
         return self.last_ehlo_response is None and self.last_helo_response is None
 
     @property
-    def supported_auth_methods(self) -> List[str]:
+    def supported_auth_methods(self) -> list[str]:
         """
         Get all AUTH methods supported by the both server and by us.
         """
@@ -236,56 +229,62 @@ class SMTP:
 
     def _update_settings_from_kwargs(
         self,
-        hostname: Optional[Union[str, Default]] = _default,
-        port: Optional[Union[int, Default]] = _default,
-        username: Optional[Union[str, bytes, Default]] = _default,
-        password: Optional[Union[str, bytes, Default]] = _default,
-        local_hostname: Optional[Union[str, Default]] = _default,
-        source_address: Optional[Union[Tuple[str, int], Default]] = _default,
+        hostname: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        port: Optional[Union[int, Literal[Default.token]]] = Default.token,
+        username: Optional[Union[str, bytes, Literal[Default.token]]] = Default.token,
+        password: Optional[Union[str, bytes, Literal[Default.token]]] = Default.token,
+        local_hostname: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        source_address: Optional[
+            Union[tuple[str, int], Literal[Default.token]]
+        ] = Default.token,
         use_tls: Optional[bool] = None,
-        start_tls: Optional[Union[bool, Default]] = _default,
+        start_tls: Optional[Union[bool, Literal[Default.token]]] = Default.token,
         validate_certs: Optional[bool] = None,
-        client_cert: Optional[Union[str, Default]] = _default,
-        client_key: Optional[Union[str, Default]] = _default,
-        tls_context: Optional[Union[ssl.SSLContext, Default]] = _default,
-        cert_bundle: Optional[Union[str, Default]] = _default,
-        socket_path: Optional[Union[SocketPathType, Default]] = _default,
-        sock: Optional[Union[socket.socket, Default]] = _default,
+        client_cert: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        client_key: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        tls_context: Optional[
+            Union[ssl.SSLContext, Literal[Default.token]]
+        ] = Default.token,
+        cert_bundle: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        socket_path: Optional[
+            Union[SocketPathType, Literal[Default.token]]
+        ] = Default.token,
+        sock: Optional[Union[socket.socket, Literal[Default.token]]] = Default.token,
     ) -> None:
         """Update our configuration from the kwargs provided.
 
         This method can be called multiple times.
         """
-        if hostname is not _default:
+        if hostname is not Default.token:
             self.hostname = hostname
         if use_tls is not None:
             self.use_tls = use_tls
-        if start_tls is not _default:
+        if start_tls is not Default.token:
             self._start_tls_on_connect = start_tls
         if validate_certs is not None:
             self.validate_certs = validate_certs
-        if port is not _default:
+        if port is not Default.token:
             self.port = port
-        if username is not _default:
+        if username is not Default.token:
             self._login_username = username
-        if password is not _default:
+        if password is not Default.token:
             self._login_password = password
 
-        if local_hostname is not _default:
-            self._local_hostname = local_hostname
-        if source_address is not _default:
+        if local_hostname is not Default.token:
+            self.local_hostname = local_hostname
+        if source_address is not Default.token:
             self.source_address = source_address
-        if client_cert is not _default:
+        if client_cert is not Default.token:
             self.client_cert = client_cert
-        if client_key is not _default:
+        if client_key is not Default.token:
             self.client_key = client_key
-        if tls_context is not _default:
+        if tls_context is not Default.token:
             self.tls_context = tls_context
-        if cert_bundle is not _default:
+        if cert_bundle is not Default.token:
             self.cert_bundle = cert_bundle
-        if socket_path is not _default:
+        if socket_path is not Default.token:
             self.socket_path = socket_path
-        if sock is not _default:
+        if sock is not Default.token:
             self.sock = sock
 
     def _validate_config(self) -> None:
@@ -297,18 +296,19 @@ class SMTP:
                 "Either a TLS context or a certificate/key must be provided"
             )
 
-        if self.sock is not None and any([self.hostname, self.port, self.socket_path]):
+        if self.sock is not None and self.socket_path is not None:
             raise ValueError(
-                "The socket option is not compatible with hostname, port or socket_path"
+                "Either a socket or a socket path must be provided, not both"
             )
 
-        if self.socket_path is not None and any([self.hostname, self.port]):
-            raise ValueError(
-                "The socket_path option is not compatible with hostname/port"
-            )
+        if (self.sock or self.socket_path) and self.port is not None:
+            raise ValueError("If using a socket, port is not required")
 
-        if self._local_hostname is not None and (
-            "\r" in self._local_hostname or "\n" in self._local_hostname
+        if (self.sock or self.socket_path) and self.use_tls and self.hostname is None:
+            raise ValueError("If using a socket with TLS, hostname is required")
+
+        if self.local_hostname is not None and (
+            "\r" in self.local_hostname or "\n" in self.local_hostname
         ):
             raise ValueError(
                 "The local_hostname param contains prohibited newline characters"
@@ -332,44 +332,52 @@ class SMTP:
 
         return SMTP_PORT
 
+    async def _get_default_local_hostname(self) -> str:
+        return await asyncio.to_thread(socket.getfqdn)
+
     async def connect(
         self,
         *,
-        hostname: Optional[Union[str, Default]] = _default,
-        port: Optional[Union[int, Default]] = _default,
-        username: Optional[Union[str, bytes, Default]] = _default,
-        password: Optional[Union[str, bytes, Default]] = _default,
-        local_hostname: Optional[Union[str, Default]] = _default,
-        source_address: Optional[Union[Tuple[str, int], Default]] = _default,
-        timeout: Optional[Union[float, Default]] = _default,
+        hostname: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        port: Optional[Union[int, Literal[Default.token]]] = Default.token,
+        username: Optional[Union[str, bytes, Literal[Default.token]]] = Default.token,
+        password: Optional[Union[str, bytes, Literal[Default.token]]] = Default.token,
+        local_hostname: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        source_address: Optional[
+            Union[tuple[str, int], Literal[Default.token]]
+        ] = Default.token,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
         use_tls: Optional[bool] = None,
-        start_tls: Optional[Union[bool, Default]] = _default,
+        start_tls: Optional[Union[bool, Literal[Default.token]]] = Default.token,
         validate_certs: Optional[bool] = None,
-        client_cert: Optional[Union[str, Default]] = _default,
-        client_key: Optional[Union[str, Default]] = _default,
-        tls_context: Optional[Union[ssl.SSLContext, Default]] = _default,
-        cert_bundle: Optional[Union[str, Default]] = _default,
-        socket_path: Optional[Union[SocketPathType, Default]] = _default,
-        sock: Optional[Union[socket.socket, Default]] = _default,
+        client_cert: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        client_key: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        tls_context: Optional[
+            Union[ssl.SSLContext, Literal[Default.token]]
+        ] = Default.token,
+        cert_bundle: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        socket_path: Optional[
+            Union[SocketPathType, Literal[Default.token]]
+        ] = Default.token,
+        sock: Optional[Union[socket.socket, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Initialize a connection to the server. Options provided to
         :meth:`.connect` take precedence over those used to initialize the
         class.
 
-        :keyword hostname:  Server name (or IP) to connect to. Defaults to "localhost".
-        :keyword port: Server port. Defaults ``465`` if ``use_tls`` is ``True``,
+        :keyword hostname:  Server name (or IP) to connect to. defaults to "localhost".
+        :keyword port: Server port. defaults ``465`` if ``use_tls`` is ``True``,
             ``587`` if ``start_tls`` is ``True``, or ``25`` otherwise.
         :keyword username:  Username to login as after connect.
         :keyword password:  Password for login after connect.
         :keyword local_hostname: The hostname of the client.  If specified, used as the
             FQDN of the local host in the HELO/EHLO command. Otherwise, the result of
-            :func:`socket.getfqdn`. **Note that getfqdn will block the event loop.**
+            :func:`socket.getfqdn`.
         :keyword source_address: Takes a 2-tuple (host, port) for the socket to bind to
             as its source address before connecting. If the host is '' and port is 0,
             the OS default behavior will be used.
-        :keyword timeout: Default timeout value for the connection, in seconds.
-            Defaults to 60.
+        :keyword timeout: timeout value for the connection, in seconds. Defaults to 60.
         :keyword use_tls: If True, make the initial connection to the server
             over TLS/SSL. Mutually exclusive with ``start_tls``; if the server uses
             STARTTLS, ``use_tls`` should be ``False``.
@@ -380,16 +388,18 @@ class SMTP:
             If ``False``, no upgrade will occur.
             Mutually exclusive with ``use_tls``.
         :keyword validate_certs: Determines if server certificates are
-            validated. Defaults to ``True``.
+            validated. defaults to ``True``.
         :keyword client_cert: Path to client side certificate, for TLS.
         :keyword client_key: Path to client side key, for TLS.
         :keyword tls_context: An existing :py:class:`ssl.SSLContext`, for TLS.
             Mutually exclusive with ``client_cert``/``client_key``.
         :keyword cert_bundle: Path to certificate bundle, for TLS verification.
         :keyword socket_path: Path to a Unix domain socket. Not compatible with
-            hostname or port. Accepts str, bytes, or a pathlike object.
-        :keyword sock: An existing, connected socket object. If given, none of
-            hostname, port, or socket_path should be provided.
+            `port` or `sock`. Accepts str, bytes, or a pathlike object.
+        :keyword sock: An existing, connected socket object. Not compatible with
+            `port`, or `socket_path`. Passing a socket object will transfer
+            control of it to the asyncio connection, and it will be closed when
+            the client disconnects.
 
         :raises ValueError: mutually exclusive options provided
         """
@@ -417,14 +427,20 @@ class SMTP:
             self._connect_lock = asyncio.Lock()
         await self._connect_lock.acquire()
 
-        # Set default port last in case use_tls or start_tls is provided,
-        # and only if we're not using a socket.
-        if self.port is None and self.sock is None and self.socket_path is None:
-            self.port = self._get_default_port()
+        # If we're not using a socket, default to port and hostname
+        if self.sock is None and self.socket_path is None:
+            if self.hostname is None:
+                self.hostname = "localhost"
+
+            if self.port is None and self.sock is None and self.socket_path is None:
+                self.port = self._get_default_port()
+
+        if self.local_hostname is None:
+            self.local_hostname = await self._get_default_local_hostname()
 
         try:
             response = await self._create_connection(
-                timeout=self.timeout if timeout is _default else timeout
+                timeout=self.timeout if timeout is Default.token else timeout
             )
             await self._maybe_start_tls_on_connect()
             await self._maybe_login_on_connect()
@@ -442,15 +458,18 @@ class SMTP:
 
         tls_context: Optional[ssl.SSLContext] = None
         ssl_handshake_timeout: Optional[float] = None
+        server_hostname: Optional[str] = None
         if self.use_tls:
             tls_context = self._get_tls_context()
             ssl_handshake_timeout = timeout
+            server_hostname = self.hostname
 
         if self.sock is not None:
             connect_coro = self.loop.create_connection(
                 lambda: protocol,
                 sock=self.sock,
                 ssl=tls_context,
+                server_hostname=server_hostname,
                 ssl_handshake_timeout=ssl_handshake_timeout,
             )
         elif self.socket_path is not None:
@@ -458,6 +477,7 @@ class SMTP:
                 lambda: protocol,
                 path=self.socket_path,  # type: ignore
                 ssl=tls_context,
+                server_hostname=server_hostname,
                 ssl_handshake_timeout=ssl_handshake_timeout,
             )
         else:
@@ -505,9 +525,6 @@ class SMTP:
 
         return response
 
-    def _connection_lost(self, waiter: "asyncio.Future[None]") -> None:
-        self.close()
-
     async def _maybe_start_tls_on_connect(self) -> None:
         """
         Depending on config, upgrade the connection via STARTTLS.
@@ -534,7 +551,9 @@ class SMTP:
             await self.login(self._login_username, login_password)
 
     async def execute_command(
-        self, *args: bytes, timeout: Optional[Union[float, Default]] = _default
+        self,
+        *args: bytes,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Check that we're connected, if we got a timeout value, and then
@@ -545,9 +564,13 @@ class SMTP:
         if self.protocol is None:
             raise SMTPServerDisconnected("Server not connected")
 
-        response = await self.protocol.execute_command(
-            *args, timeout=self.timeout if timeout is _default else timeout
-        )
+        try:
+            response = await self.protocol.execute_command(
+                *args, timeout=self.timeout if timeout is Default.token else timeout
+            )
+        except SMTPServerDisconnected:
+            self.close()
+            raise
 
         # If the server is unavailable, be nice and close the connection
         if response.code == SMTPStatus.domain_unavailable:
@@ -621,7 +644,7 @@ class SMTP:
         self,
         *,
         hostname: Optional[str] = None,
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Send the SMTP HELO command.
@@ -630,8 +653,15 @@ class SMTP:
 
         :raises SMTPHeloError: on unexpected server response code
         """
+        if hostname is None:
+            # Should already be set on connect
+            if self.local_hostname is None:
+                self.local_hostname = await self._get_default_local_hostname()
+
+            hostname = self.local_hostname
+
         response = self.last_helo_response = await self.execute_command(
-            b"HELO", (hostname or self.local_hostname).encode("ascii"), timeout=timeout
+            b"HELO", hostname.encode("ascii"), timeout=timeout
         )
 
         if response.code != SMTPStatus.completed:
@@ -639,7 +669,9 @@ class SMTP:
 
         return response
 
-    async def help(self, *, timeout: Optional[Union[float, Default]] = _default) -> str:
+    async def help(
+        self, *, timeout: Optional[Union[float, Literal[Default.token]]] = Default.token
+    ) -> str:
         """
         Send the SMTP HELP command, which responds with help text.
 
@@ -658,7 +690,7 @@ class SMTP:
         return response.message
 
     async def rset(
-        self, *, timeout: Optional[Union[float, Default]] = _default
+        self, *, timeout: Optional[Union[float, Literal[Default.token]]] = Default.token
     ) -> SMTPResponse:
         """
         Send an SMTP RSET command, which resets the server's envelope
@@ -675,7 +707,7 @@ class SMTP:
         return response
 
     async def noop(
-        self, *, timeout: Optional[Union[float, Default]] = _default
+        self, *, timeout: Optional[Union[float, Literal[Default.token]]] = Default.token
     ) -> SMTPResponse:
         """
         Send an SMTP NOOP command, which does nothing.
@@ -696,7 +728,7 @@ class SMTP:
         /,
         *,
         options: Optional[Iterable[str]] = None,
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Send an SMTP VRFY command, which tests an address for validity.
@@ -737,7 +769,7 @@ class SMTP:
         /,
         *,
         options: Optional[Iterable[str]] = None,
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Send an SMTP EXPN command, which expands a mailing list.
@@ -769,7 +801,7 @@ class SMTP:
         return response
 
     async def quit(
-        self, *, timeout: Optional[Union[float, Default]] = _default
+        self, *, timeout: Optional[Union[float, Literal[Default.token]]] = Default.token
     ) -> SMTPResponse:
         """
         Send the SMTP QUIT command, which closes the connection.
@@ -792,7 +824,7 @@ class SMTP:
         *,
         options: Optional[Iterable[str]] = None,
         encoding: str = "ascii",
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Send an SMTP MAIL command, which specifies the message sender and
@@ -825,7 +857,7 @@ class SMTP:
         *,
         options: Optional[Iterable[str]] = None,
         encoding: str = "ascii",
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Send an SMTP RCPT command, which specifies a single recipient for
@@ -857,7 +889,7 @@ class SMTP:
         message: Union[str, bytes],
         /,
         *,
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Send an SMTP DATA command, followed by the message given.
@@ -866,19 +898,22 @@ class SMTP:
         :raises SMTPDataError: on unexpected server response code
         :raises SMTPServerDisconnected: connection lost
         """
-        await self._ehlo_or_helo_if_needed()
-
-        # As data accesses protocol directly, some handling is required
         if self.protocol is None:
             raise SMTPServerDisconnected("Connection lost")
 
-        if timeout is _default:
+        await self._ehlo_or_helo_if_needed()
+
+        if timeout is Default.token:
             timeout = self.timeout
 
         if isinstance(message, str):
             message = message.encode("ascii")
 
-        return await self.protocol.execute_data_command(message, timeout=timeout)
+        try:
+            return await self.protocol.execute_data_command(message, timeout=timeout)
+        except SMTPServerDisconnected:
+            self.close()
+            raise
 
     # ESMTP commands #
 
@@ -886,7 +921,7 @@ class SMTP:
         self,
         *,
         hostname: Optional[str] = None,
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Send the SMTP EHLO command.
@@ -896,6 +931,10 @@ class SMTP:
         :raises SMTPHeloError: on unexpected server response code
         """
         if hostname is None:
+            # Should already be set on connect
+            if self.local_hostname is None:
+                self.local_hostname = await self._get_default_local_hostname()
+
             hostname = self.local_hostname
 
         response = await self.execute_command(
@@ -945,11 +984,13 @@ class SMTP:
         *,
         server_hostname: Optional[str] = None,
         validate_certs: Optional[bool] = None,
-        client_cert: Optional[Union[str, Default]] = _default,
-        client_key: Optional[Union[str, Default]] = _default,
-        cert_bundle: Optional[Union[str, Default]] = _default,
-        tls_context: Optional[Union[ssl.SSLContext, Default]] = _default,
-        timeout: Optional[Union[float, Default]] = _default,
+        client_cert: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        client_key: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        cert_bundle: Optional[Union[str, Literal[Default.token]]] = Default.token,
+        tls_context: Optional[
+            Union[ssl.SSLContext, Literal[Default.token]]
+        ] = Default.token,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Puts the connection to the SMTP server into TLS mode.
@@ -969,14 +1010,14 @@ class SMTP:
         :raises SMTPServerDisconnected: connection lost
         :raises ValueError: invalid options provided
         """
-        await self._ehlo_or_helo_if_needed()
-
         if self.protocol is None:
             raise SMTPServerDisconnected("Server not connected")
 
         if self.get_transport_info("sslcontext") is not None:
             raise SMTPException("Connection already using TLS")
 
+        await self._ehlo_or_helo_if_needed()
+
         self._update_settings_from_kwargs(
             validate_certs=validate_certs,
             client_cert=client_cert,
@@ -989,7 +1030,7 @@ class SMTP:
         if server_hostname is None:
             server_hostname = self.hostname
 
-        if timeout is _default:
+        if timeout is Default.token:
             timeout = self.timeout
 
         tls_context = self._get_tls_context()
@@ -997,11 +1038,14 @@ class SMTP:
         if not self.supports_extension("starttls"):
             raise SMTPException("SMTP STARTTLS extension not supported by server.")
 
-        response = await self.protocol.start_tls(
-            tls_context, server_hostname=server_hostname, timeout=timeout
-        )
-        if self.protocol is None:
-            raise SMTPServerDisconnected("Connection lost")
+        try:
+            response = await self.protocol.start_tls(
+                tls_context, server_hostname=server_hostname, timeout=timeout
+            )
+        except SMTPServerDisconnected:
+            self.close()
+            raise
+
         # Update our transport reference
         self.transport = self.protocol.transport
 
@@ -1020,7 +1064,7 @@ class SMTP:
         username: Union[str, bytes],
         password: Union[str, bytes],
         /,
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         Tries to login with supported auth methods.
@@ -1044,7 +1088,7 @@ class SMTP:
         response: Optional[SMTPResponse] = None
         exception: Optional[SMTPAuthenticationError] = None
         for auth_name in self.supported_auth_methods:
-            method_name = f'auth_{auth_name.replace("-", "")}'
+            method_name = f"auth_{auth_name.replace('-', '')}"
             try:
                 auth_method = getattr(self, method_name)
             except AttributeError as err:
@@ -1070,7 +1114,7 @@ class SMTP:
         password: Union[str, bytes],
         /,
         *,
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         CRAM-MD5 auth uses the password as a shared secret to MD5 the server's
@@ -1109,7 +1153,7 @@ class SMTP:
         password: Union[str, bytes],
         /,
         *,
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         PLAIN auth encodes the username and password in one Base64 encoded
@@ -1138,7 +1182,7 @@ class SMTP:
         password: Union[str, bytes],
         /,
         *,
-        timeout: Optional[Union[float, Default]] = _default,
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
     ) -> SMTPResponse:
         """
         LOGIN auth sends the Base64 encoded username and password in sequence.
@@ -1189,8 +1233,8 @@ class SMTP:
         *,
         mail_options: Optional[Iterable[str]] = None,
         rcpt_options: Optional[Iterable[str]] = None,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> Tuple[Dict[str, SMTPResponse], str]:
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
+    ) -> tuple[dict[str, SMTPResponse], str]:
         """
         This command performs an entire mail transaction.
 
@@ -1222,17 +1266,15 @@ class SMTP:
 
         Example:
 
-             >>> event_loop = asyncio.get_event_loop()
-             >>> smtp = aiosmtplib.SMTP(hostname="127.0.0.1", port=1025)
-             >>> event_loop.run_until_complete(smtp.connect())
-             (220, ...)
-             >>> recipients = ["one@one.org", "two@two.org", "3@three.org"]
-             >>> message = "From: Me@my.org\\nSubject: testing\\nHello World"
-             >>> send_coro = smtp.sendmail("me@my.org", recipients, message)
-             >>> event_loop.run_until_complete(send_coro)
-             ({}, 'OK')
-             >>> event_loop.run_until_complete(smtp.quit())
-             (221, Bye)
+            >>> smtp = aiosmtplib.SMTP(hostname="127.0.0.1", port=1025)
+            >>> recipients = ["one@one.org", "two@two.org", "3@three.org"]
+            >>> message = "From: Me@my.org\\nSubject: testing\\nHello World"
+            >>> async def connect_and_send():
+            ...     await smtp.connect()
+            ...     await smtp.sendmail("me@my.org", recipients, message)
+            ...     return await smtp.quit()
+            >>> asyncio.run(connect_and_send())
+            (221, Bye)
 
         In the above example, the message was accepted for delivery for all
         three addresses. If delivery had been only successful to two
@@ -1314,13 +1356,13 @@ class SMTP:
         recipients: Sequence[str],
         options: Iterable[str],
         encoding: str = "ascii",
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> Dict[str, SMTPResponse]:
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
+    ) -> dict[str, SMTPResponse]:
         """
         Send the recipients given to the server. Used as part of
         :meth:`.sendmail`.
         """
-        recipient_errors = []
+        recipient_errors: list[SMTPRecipientRefused] = []
         for address in recipients:
             try:
                 await self.rcpt(
@@ -1348,8 +1390,8 @@ class SMTP:
         recipients: Optional[Union[str, Sequence[str]]] = None,
         mail_options: Optional[Iterable[str]] = None,
         rcpt_options: Optional[Iterable[str]] = None,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> Tuple[Dict[str, SMTPResponse], str]:
+        timeout: Optional[Union[float, Literal[Default.token]]] = Default.token,
+    ) -> tuple[dict[str, SMTPResponse], str]:
         r"""
         Sends an :py:class:`email.message.EmailMessage` object.
 
@@ -1432,13 +1474,13 @@ class SMTP:
 
     def sendmail_sync(
         self, *args: Any, **kwargs: Any
-    ) -> Tuple[Dict[str, SMTPResponse], str]:
+    ) -> tuple[dict[str, SMTPResponse], str]:
         """
         Synchronous version of :meth:`.sendmail`. This method starts
         an event loop to connect, send the message, and disconnect.
         """
 
-        async def sendmail_coroutine() -> Tuple[Dict[str, SMTPResponse], str]:
+        async def sendmail_coroutine() -> tuple[dict[str, SMTPResponse], str]:
             async with self:
                 return await self.sendmail(*args, **kwargs)
 
@@ -1446,13 +1488,13 @@ class SMTP:
 
     def send_message_sync(
         self, *args: Any, **kwargs: Any
-    ) -> Tuple[Dict[str, SMTPResponse], str]:
+    ) -> tuple[dict[str, SMTPResponse], str]:
         """
         Synchronous version of :meth:`.send_message`. This method
         starts an event loop to connect, send the message, and disconnect.
         """
 
-        async def send_message_coroutine() -> Tuple[Dict[str, SMTPResponse], str]:
+        async def send_message_coroutine() -> tuple[dict[str, SMTPResponse], str]:
             async with self:
                 return await self.send_message(*args, **kwargs)
 
