@@ -7,12 +7,12 @@ import email.header
 import email.message
 import email.mime.multipart
 import email.mime.text
-import pathlib
 import socket
 import ssl
 import sys
+from collections.abc import Callable, Generator
 from pathlib import Path
-from typing import Any, Callable, Coroutine, List, Optional, Tuple, Type, Union
+from typing import Any, Optional, Union
 
 import hypothesis
 import pytest
@@ -21,7 +21,7 @@ import trustme
 from aiosmtpd.controller import Controller as SMTPDController
 from aiosmtpd.smtp import SMTP as SMTPD
 
-from aiosmtplib import SMTP, SMTPStatus
+from aiosmtplib import SMTP
 
 from .auth import DummySMTPAuth
 from .compat import cleanup_server
@@ -80,27 +80,19 @@ original_event_loop_policy = None
 
 
 def pytest_sessionstart(session: pytest.Session) -> None:
-    global original_event_loop_policy
-
     # Install the uvloop event loop policy globally, per session
     loop_type = session.config.getoption("--event-loop")
     if loop_type == "uvloop":
         if not HAS_UVLOOP:
             raise RuntimeError("uvloop not installed.")
-        original_event_loop_policy = asyncio.get_event_loop_policy()
-        policy = uvloop.EventLoopPolicy()
-        asyncio.set_event_loop_policy(policy)
 
-
-def pytest_sessionfinish(session: pytest.Session, exitstatus: int) -> None:
-    if original_event_loop_policy is not None:
-        asyncio.set_event_loop_policy(original_event_loop_policy)
+        uvloop.install()  # type: ignore
 
 
 @pytest_asyncio.fixture
 def debug_event_loop(
     event_loop: asyncio.AbstractEventLoop,
-) -> asyncio.AbstractEventLoop:
+) -> Generator[asyncio.AbstractEventLoop, None, None]:
     previous_debug = event_loop.get_debug()
     event_loop.set_debug(True)
 
@@ -152,7 +144,7 @@ def message_str(recipient_str: str, sender_str: str) -> str:
 
 
 @pytest.fixture(scope="session")
-def smtpd_class() -> Type[SMTPD]:
+def smtpd_class() -> type[SMTPD]:
     return TestSMTPD
 
 
@@ -217,42 +209,55 @@ def server_tls_context(
     return tls_context
 
 
-@pytest.fixture(scope="function")
-def ca_cert_path(tmp_path: pathlib.Path, cert_authority: trustme.CA) -> str:
+@pytest.fixture(scope="session")
+def ca_cert_path(
+    tmp_path_factory: pytest.TempPathFactory, cert_authority: trustme.CA
+) -> str:
+    tmp_path = tmp_path_factory.mktemp("cacert")
+
     cert_authority.cert_pem.write_to_path(tmp_path / "ca.pem")
 
     return str(tmp_path / "ca.pem")
 
 
-@pytest.fixture(scope="function")
-def valid_cert_path(tmp_path: pathlib.Path, valid_client_cert: trustme.LeafCert) -> str:
+@pytest.fixture(scope="session")
+def valid_cert_path(
+    tmp_path_factory: pytest.TempPathFactory, valid_client_cert: trustme.LeafCert
+) -> str:
+    tmp_path = tmp_path_factory.mktemp("cert-valid-pem")
     for pem in valid_client_cert.cert_chain_pems:
         pem.write_to_path(tmp_path / "valid.pem", append=True)
 
     return str(tmp_path / "valid.pem")
 
 
-@pytest.fixture(scope="function")
-def valid_key_path(tmp_path: pathlib.Path, valid_client_cert: trustme.LeafCert) -> str:
+@pytest.fixture(scope="session")
+def valid_key_path(
+    tmp_path_factory: pytest.TempPathFactory, valid_client_cert: trustme.LeafCert
+) -> str:
+    tmp_path = tmp_path_factory.mktemp("cert-valid-key")
+
     valid_client_cert.private_key_pem.write_to_path(tmp_path / "valid.key")
 
     return str(tmp_path / "valid.key")
 
 
-@pytest.fixture(scope="function")
+@pytest.fixture(scope="session")
 def invalid_cert_path(
-    tmp_path: pathlib.Path, unknown_client_cert: trustme.LeafCert
+    tmp_path_factory: pytest.TempPathFactory, unknown_client_cert: trustme.LeafCert
 ) -> str:
+    tmp_path = tmp_path_factory.mktemp("cert-invalid-pem")
     for pem in unknown_client_cert.cert_chain_pems:
         pem.write_to_path(tmp_path / "invalid.pem", append=True)
 
     return str(tmp_path / "invalid.pem")
 
 
-@pytest.fixture(scope="function")
+@pytest.fixture(scope="session")
 def invalid_key_path(
-    tmp_path: pathlib.Path, unknown_client_cert: trustme.LeafCert
+    tmp_path_factory: pytest.TempPathFactory, unknown_client_cert: trustme.LeafCert
 ) -> str:
+    tmp_path = tmp_path_factory.mktemp("cert-invalid-key")
     unknown_client_cert.private_key_pem.write_to_path(tmp_path / "invalid.key")
     return str(tmp_path / "invalid.key")
 
@@ -267,28 +272,6 @@ def auth_password() -> str:
     return "test"
 
 
-# Error code params #
-
-
-@pytest.fixture(
-    scope="function",
-    params=[
-        SMTPStatus.mailbox_unavailable,
-        SMTPStatus.unrecognized_command,
-        SMTPStatus.bad_command_sequence,
-        SMTPStatus.syntax_error,
-    ],
-    ids=[
-        SMTPStatus.mailbox_unavailable.name,
-        SMTPStatus.unrecognized_command.name,
-        SMTPStatus.bad_command_sequence.name,
-        SMTPStatus.syntax_error.name,
-    ],
-)
-def error_code(request: ParamFixtureRequest) -> int:
-    return int(request.param.value)
-
-
 # Auth #
 
 
@@ -340,25 +323,25 @@ def message(
 
 
 @pytest.fixture(scope="function")
-def received_messages() -> List[email.message.EmailMessage]:
+def received_messages() -> list[email.message.EmailMessage]:
     return []
 
 
 @pytest.fixture(scope="function")
-def received_commands() -> List[Tuple[str, Tuple[Any, ...]]]:
+def received_commands() -> list[tuple[str, tuple[Any, ...]]]:
     return []
 
 
 @pytest.fixture(scope="function")
-def smtpd_responses() -> List[str]:
+def smtpd_responses() -> list[str]:
     return []
 
 
 @pytest.fixture(scope="function")
 def smtpd_handler(
-    received_messages: List[email.message.EmailMessage],
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
-    smtpd_responses: List[str],
+    received_messages: list[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
+    smtpd_responses: list[str],
 ) -> RecordingHandler:
     return RecordingHandler(received_messages, received_commands, smtpd_responses)
 
@@ -376,217 +359,6 @@ def smtpd_auth_callback(
     return auth_callback
 
 
-# Mock response #
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_delayed_ok() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_delayed_ok(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        await asyncio.sleep(1.0)
-        await smtpd.push("250 all done")
-
-    return mock_response_delayed_ok
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_delayed_read() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_delayed_read(
-        smtpd: SMTPD, *args: Any, **kwargs: Any
-    ) -> None:
-        await smtpd.push("220-hi")
-        await asyncio.sleep(1.0)
-
-    return mock_response_delayed_read
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_done() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_done(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        if args and args[0]:
-            smtpd.session.host_name = args[0]
-        await smtpd.push("250 done")
-
-    return mock_response_done
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_done_then_close() -> (
-    Callable[[SMTPD], Coroutine[Any, Any, None]]
-):
-    async def mock_response_done_then_close(
-        smtpd: SMTPD, *args: Any, **kwargs: Any
-    ) -> None:
-        if args and args[0]:
-            smtpd.session.host_name = args[0]
-        await smtpd.push("250 done")
-        await smtpd.push("221 bye now")
-        smtpd.transport.close()
-
-    return mock_response_done_then_close
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_error() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_error(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        await smtpd.push("555 error")
-
-    return mock_response_error
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_error_disconnect() -> (
-    Callable[[SMTPD], Coroutine[Any, Any, None]]
-):
-    async def mock_response_error_disconnect(
-        smtpd: SMTPD, *args: Any, **kwargs: Any
-    ) -> None:
-        await smtpd.push("501 error")
-        smtpd.transport.close()
-
-    return mock_response_error_disconnect
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_bad_data() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_bad_data(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        smtpd._writer.write(b"250 \xff\xff\xff\xff\r\n")
-        await smtpd._writer.drain()
-
-    return mock_response_bad_data
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_gibberish() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_gibberish(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        smtpd._writer.write("wefpPSwrsfa2sdfsdf")
-        await smtpd._writer.drain()
-
-    return mock_response_gibberish
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_expn() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_expn(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        await smtpd.push(
-            """250-Joseph Blow <jblow@example.com>
-250 Alice Smith <asmith@example.com>"""
-        )
-
-    return mock_response_expn
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_ehlo_minimal() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_ehlo(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        if args and args[0]:
-            smtpd.session.host_name = args[0]
-
-        await smtpd.push("250 HELP")
-
-    return mock_response_ehlo
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_ehlo_full() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_ehlo(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        if args and args[0]:
-            smtpd.session.host_name = args[0]
-
-        await smtpd.push(
-            """250-localhost
-250-PIPELINING
-250-8BITMIME
-250-SIZE 512000
-250-DSN
-250-ENHANCEDSTATUSCODES
-250-EXPN
-250-HELP
-250-SAML
-250-SEND
-250-SOML
-250-TURN
-250-XADR
-250-XSTA
-250-ETRN
-250 XGEN"""
-        )
-
-    return mock_response_ehlo
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_unavailable() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_unavailable(
-        smtpd: SMTPD, *args: Any, **kwargs: Any
-    ) -> None:
-        await smtpd.push("421 retry in 5 minutes")
-        smtpd.transport.close()
-
-    return mock_response_unavailable
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_tls_not_available() -> (
-    Callable[[SMTPD], Coroutine[Any, Any, None]]
-):
-    async def mock_tls_not_available(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        await smtpd.push("454 please login")
-
-    return mock_tls_not_available
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_tls_ready_disconnect() -> (
-    Callable[[SMTPD], Coroutine[Any, Any, None]]
-):
-    async def mock_response_tls_ready_disconnect(
-        smtpd: SMTPD, *args: Any, **kwargs: Any
-    ) -> None:
-        await smtpd.push("220 go for it")
-        smtpd.transport.close()
-
-    return mock_response_tls_ready_disconnect
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_disconnect() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_disconnect(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        smtpd.transport.close()
-
-    return mock_response_disconnect
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_eof() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    async def mock_response_eof(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        smtpd.transport.write_eof()
-
-    return mock_response_eof
-
-
-@pytest.fixture(scope="session")
-def smtpd_mock_response_error_with_code_factory() -> (
-    Callable[[str], Callable[[SMTPD], Coroutine[Any, Any, None]]]
-):
-    def factory(error_code: str) -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-        async def mock_error_response(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-            await smtpd.push(f"{error_code} error")
-
-        return mock_error_response
-
-    return factory
-
-
-@pytest.fixture(scope="function")
-def smtpd_mock_response_error_with_code(
-    error_code: int,
-    smtpd_mock_response_error_with_code_factory: Callable[
-        [str], Callable[[SMTPD], Coroutine[Any, Any, None]]
-    ],
-) -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
-    return smtpd_mock_response_error_with_code_factory(str(error_code))
-
-
 @pytest.fixture(
     scope="function",
     params=(str, bytes, Path),
@@ -615,202 +387,149 @@ def socket_path(
 # Servers #
 
 
+@pytest.fixture(scope="function")
+def smtpd_factory(
+    request: pytest.FixtureRequest,
+    monkeypatch: pytest.MonkeyPatch,
+    hostname: str,
+    smtpd_handler: RecordingHandler,
+    server_tls_context: ssl.SSLContext,
+    smtpd_auth_callback: Callable[[str, bytes, bytes], bool],
+) -> Callable[[], SMTPD]:
+    smtpd_options_marker = request.node.get_closest_marker("smtpd_options")
+    if smtpd_options_marker is None:
+        smtpd_options = {}
+    else:
+        smtpd_options = smtpd_options_marker.kwargs
+
+    smtpd_mocks_marker = request.node.get_closest_marker("smtpd_mocks")
+    if smtpd_mocks_marker is None:
+        smtpd_mocks = {}
+    else:
+        smtpd_mocks = smtpd_mocks_marker.kwargs
+
+    for attr, mock_fn in smtpd_mocks.items():
+        monkeypatch.setattr(TestSMTPD, attr, mock_fn)
+
+    smtpd_tls_context = (
+        server_tls_context
+        if smtpd_options.get("starttls", True) or smtpd_options.get("tls", False)
+        else None
+    )
+
+    def factory() -> SMTPD:
+        return TestSMTPD(
+            smtpd_handler,
+            hostname=hostname,
+            enable_SMTPUTF8=smtpd_options.get("smtputf8", False),
+            decode_data=smtpd_options.get("7bit", False),
+            tls_context=smtpd_tls_context,
+            auth_callback=smtpd_auth_callback,
+        )
+
+    return factory
+
+
 @pytest.fixture(scope="function")
 def smtpd_server(
     request: pytest.FixtureRequest,
     event_loop: asyncio.AbstractEventLoop,
     bind_address: str,
-    hostname: str,
-    smtpd_class: Type[SMTPD],
-    smtpd_handler: RecordingHandler,
     server_tls_context: ssl.SSLContext,
-    smtpd_auth_callback: Callable[[str, bytes, bytes], bool],
-) -> asyncio.AbstractServer:
-    def factory() -> SMTPD:
-        return smtpd_class(
-            smtpd_handler,
-            hostname=hostname,
-            enable_SMTPUTF8=False,
-            tls_context=server_tls_context,
-            auth_callback=smtpd_auth_callback,
-        )
+    smtpd_factory: Callable[[], SMTPD],
+) -> Generator[asyncio.AbstractServer, None, None]:
+    smtpd_options_marker = request.node.get_closest_marker("smtpd_options")
+    if smtpd_options_marker is None:
+        smtpd_options = {}
+    else:
+        smtpd_options = smtpd_options_marker.kwargs
 
-    server = event_loop.run_until_complete(
-        event_loop.create_server(
-            factory, host=bind_address, port=0, family=socket.AF_INET
-        )
-    )
+    create_server_kwargs = {
+        "host": bind_address,
+        "port": 0,
+        "family": socket.AF_INET,
+    }
+    if smtpd_options.get("tls", False):
+        create_server_kwargs["ssl"] = server_tls_context
 
-    def close_server() -> None:
-        server.close()
-        try:
-            event_loop.run_until_complete(cleanup_server(server))
-        except RuntimeError:
-            pass
+    server_coro = event_loop.create_server(smtpd_factory, **create_server_kwargs)
+    server = event_loop.run_until_complete(server_coro)
 
-    request.addfinalizer(close_server)
+    yield server
 
-    return server
-
-
-@pytest.fixture(scope="function")
-def smtpd_server_smtputf8(
-    request: pytest.FixtureRequest,
-    event_loop: asyncio.AbstractEventLoop,
-    bind_address: str,
-    hostname: str,
-    smtpd_class: Type[SMTPD],
-    smtpd_handler: RecordingHandler,
-    server_tls_context: ssl.SSLContext,
-    smtpd_auth_callback: Callable[[str, bytes, bytes], bool],
-) -> asyncio.AbstractServer:
-    def factory() -> SMTPD:
-        return smtpd_class(
-            smtpd_handler,
-            hostname=hostname,
-            enable_SMTPUTF8=True,
-            tls_context=server_tls_context,
-            auth_callback=smtpd_auth_callback,
-        )
-
-    server = event_loop.run_until_complete(
-        event_loop.create_server(
-            factory, host=bind_address, port=0, family=socket.AF_INET
-        )
-    )
-
-    def close_server() -> None:
-        server.close()
-        try:
-            event_loop.run_until_complete(cleanup_server(server))
-        except RuntimeError:
-            pass
-
-    request.addfinalizer(close_server)
-
-    return server
+    server.close()
+    try:
+        event_loop.run_until_complete(cleanup_server(server))
+    except RuntimeError:
+        pass
 
 
 @pytest.fixture(scope="function")
 def echo_server(
-    request: pytest.FixtureRequest,
-    bind_address: str,
-    event_loop: asyncio.AbstractEventLoop,
-) -> asyncio.AbstractServer:
-    server = event_loop.run_until_complete(
-        event_loop.create_server(
-            EchoServerProtocol, host=bind_address, port=0, family=socket.AF_INET
-        )
+    event_loop: asyncio.AbstractEventLoop, bind_address: str
+) -> Generator[asyncio.AbstractServer, None, None]:
+    server_coro = event_loop.create_server(
+        EchoServerProtocol, host=bind_address, port=0, family=socket.AF_INET
     )
+    server = event_loop.run_until_complete(server_coro)
 
-    def close_server() -> None:
-        server.close()
-        try:
-            event_loop.run_until_complete(cleanup_server(server))
-        except RuntimeError:
-            pass
+    yield server
 
-    return server
+    server.close()
+    try:
+        event_loop.run_until_complete(cleanup_server(server))
+    except RuntimeError:
+        pass
 
 
 @pytest.fixture(scope="function")
 def smtpd_server_socket_path(
     request: pytest.FixtureRequest,
     event_loop: asyncio.AbstractEventLoop,
-    hostname: str,
     socket_path: Union[str, bytes, Path],
-    smtpd_class: Type[SMTPD],
-    smtpd_handler: RecordingHandler,
     server_tls_context: ssl.SSLContext,
-    smtpd_auth_callback: Callable[[str, bytes, bytes], bool],
-) -> asyncio.AbstractServer:
-    def factory() -> SMTPD:
-        return smtpd_class(
-            smtpd_handler,
-            hostname=hostname,
-            enable_SMTPUTF8=False,
-            tls_context=server_tls_context,
-            auth_callback=smtpd_auth_callback,
-        )
+    smtpd_factory: Callable[[], SMTPD],
+) -> Generator[asyncio.AbstractServer, None, None]:
+    smtpd_options_marker = request.node.get_closest_marker("smtpd_options")
+    if smtpd_options_marker is None:
+        smtpd_options = {}
+    else:
+        smtpd_options = smtpd_options_marker.kwargs
 
     create_server_coro = event_loop.create_unix_server(
-        factory,
+        smtpd_factory,
         path=socket_path,  # type: ignore
+        ssl=server_tls_context if smtpd_options.get("tls", False) else None,
     )
     server = event_loop.run_until_complete(create_server_coro)
 
-    def close_server() -> None:
-        server.close()
-        try:
-            event_loop.run_until_complete(cleanup_server(server))
-        except RuntimeError:
-            pass
+    yield server
 
-    request.addfinalizer(close_server)
-
-    return server
-
-
-@pytest.fixture(scope="function")
-def smtpd_server_tls(
-    request: pytest.FixtureRequest,
-    event_loop: asyncio.AbstractEventLoop,
-    bind_address: str,
-    smtpd_class: Type[SMTPD],
-    smtpd_handler: RecordingHandler,
-    server_tls_context: ssl.SSLContext,
-) -> asyncio.AbstractServer:
-    def factory() -> SMTPD:
-        return smtpd_class(
-            smtpd_handler,
-            hostname=bind_address,
-            enable_SMTPUTF8=False,
-            tls_context=server_tls_context,
-        )
-
-    server = event_loop.run_until_complete(
-        event_loop.create_server(
-            factory,
-            host=bind_address,
-            port=0,
-            ssl=server_tls_context,
-            family=socket.AF_INET,
-        )
-    )
-
-    def close_server() -> None:
-        server.close()
-        try:
-            event_loop.run_until_complete(cleanup_server(server))
-        except RuntimeError:
-            pass
-
-    request.addfinalizer(close_server)
-
-    return server
+    server.close()
+    try:
+        event_loop.run_until_complete(cleanup_server(server))
+    except RuntimeError:
+        pass
 
 
 @pytest.fixture(scope="function")
 def smtpd_controller(
-    request: pytest.FixtureRequest,
     bind_address: str,
     unused_tcp_port: int,
     smtpd_handler: RecordingHandler,
-) -> SMTPDController:
+) -> Generator[SMTPDController, None, None]:
     port = unused_tcp_port
     controller: Optional[SMTPDController]
     controller = SMTPDController(smtpd_handler, hostname=bind_address, port=port)
     controller.start()
 
-    request.addfinalizer(controller.stop)
+    yield controller
 
-    return controller
+    controller.stop()
 
 
 @pytest.fixture(scope="function")
-def smtpd_server_threaded(
-    request: pytest.FixtureRequest, smtpd_controller: SMTPDController
-) -> asyncio.AbstractServer:
+def smtpd_server_threaded(smtpd_controller: SMTPDController) -> asyncio.AbstractServer:
     server: asyncio.AbstractServer = smtpd_controller.server
     return server
 
@@ -818,34 +537,14 @@ def smtpd_server_threaded(
 # Running server ports #
 
 
-def _get_server_socket_port(server: asyncio.AbstractServer) -> Optional[int]:
-    sockets = getattr(server, "sockets", [])
-    if sockets:
-        return int(sockets[0].getsockname()[1])
-
-    return None
+@pytest.fixture(scope="function")
+def smtpd_server_port(smtpd_server: asyncio.Server) -> int:
+    return int(smtpd_server.sockets[0].getsockname()[1])
 
 
 @pytest.fixture(scope="function")
-def smtpd_server_port(smtpd_server: asyncio.AbstractServer) -> Optional[int]:
-    return _get_server_socket_port(smtpd_server)
-
-
-@pytest.fixture(scope="function")
-def smtpd_server_smtputf8_port(
-    smtpd_server_smtputf8: asyncio.AbstractServer,
-) -> Optional[int]:
-    return _get_server_socket_port(smtpd_server_smtputf8)
-
-
-@pytest.fixture(scope="function")
-def echo_server_port(echo_server: asyncio.AbstractServer) -> Optional[int]:
-    return _get_server_socket_port(echo_server)
-
-
-@pytest.fixture(scope="function")
-def smtpd_server_tls_port(smtpd_server_tls: asyncio.AbstractServer) -> Optional[int]:
-    return _get_server_socket_port(smtpd_server_tls)
+def echo_server_port(echo_server: asyncio.Server) -> int:
+    return int(echo_server.sockets[0].getsockname()[1])
 
 
 @pytest.fixture(scope="function")
@@ -859,39 +558,25 @@ def smtpd_server_threaded_port(smtpd_controller: SMTPDController) -> int:
 
 @pytest.fixture(scope="function")
 def smtp_client(
-    hostname: str, smtpd_server_port: int, client_tls_context: ssl.SSLContext
+    request: pytest.FixtureRequest,
+    hostname: str,
+    smtpd_server_port: int,
+    client_tls_context: ssl.SSLContext,
 ) -> SMTP:
+    smtp_client_options_marker = request.node.get_closest_marker("smtp_client_options")
+    if smtp_client_options_marker is None:
+        smtp_client_options = {}
+    else:
+        smtp_client_options = smtp_client_options_marker.kwargs
+
+    smtp_client_options.setdefault("tls_context", client_tls_context)
+    smtp_client_options.setdefault("start_tls", False)
+
     return SMTP(
         hostname=hostname,
         port=smtpd_server_port,
-        tls_context=client_tls_context,
-        start_tls=False,
         timeout=1.0,
-    )
-
-
-@pytest.fixture(scope="function")
-def smtp_client_smtputf8(
-    hostname: str, smtpd_server_smtputf8_port: int, client_tls_context: ssl.SSLContext
-) -> SMTP:
-    return SMTP(
-        hostname=hostname,
-        port=smtpd_server_smtputf8_port,
-        timeout=1.0,
-        start_tls=False,
-        tls_context=client_tls_context,
-    )
-
-
-@pytest.fixture(scope="function")
-def smtp_client_tls(
-    hostname: str, smtpd_server_tls_port: int, client_tls_context: ssl.SSLContext
-) -> SMTP:
-    return SMTP(
-        hostname=hostname,
-        port=smtpd_server_tls_port,
-        use_tls=True,
-        tls_context=client_tls_context,
+        **smtp_client_options,
     )
 
 
