@@ -4,24 +4,22 @@ send coroutine testing.
 
 import asyncio
 import email
+import email.message
 import pathlib
 import socket
 import ssl
-from typing import Any, List, Tuple, Union
+from typing import Any, Union
 
 import pytest
 
 from aiosmtplib import send
 
 
-pytestmark = pytest.mark.asyncio()
-
-
 async def test_send(
     hostname: str,
     smtpd_server_port: int,
     message: email.message.Message,
-    received_messages: List[email.message.EmailMessage],
+    received_messages: list[email.message.EmailMessage],
     client_tls_context: ssl.SSLContext,
 ) -> None:
     errors, response = await send(
@@ -41,7 +39,7 @@ async def test_send_with_str(
     recipient_str: str,
     sender_str: str,
     message_str: str,
-    received_messages: List[email.message.EmailMessage],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     errors, response = await send(
         message_str,
@@ -62,7 +60,7 @@ async def test_send_with_bytes(
     recipient_str: str,
     sender_str: str,
     message_str: str,
-    received_messages: List[email.message.EmailMessage],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     errors, response = await send(
         bytes(message_str, "ascii"),
@@ -82,7 +80,7 @@ async def test_send_without_sender(
     smtpd_server_port: int,
     recipient_str: str,
     message_str: str,
-    received_messages: List[email.message.EmailMessage],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     with pytest.raises(ValueError):
         errors, response = await send(
@@ -100,7 +98,7 @@ async def test_send_without_recipients(
     smtpd_server_port: int,
     sender_str: str,
     message_str: str,
-    received_messages: List[email.message.EmailMessage],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     with pytest.raises(ValueError):
         errors, response = await send(
@@ -118,8 +116,8 @@ async def test_send_with_start_tls(
     smtpd_server_port: int,
     client_tls_context: ssl.SSLContext,
     message: email.message.Message,
-    received_messages: List[email.message.EmailMessage],
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
 ) -> None:
     errors, response = await send(
         message,
@@ -138,8 +136,8 @@ async def test_send_with_login(
     hostname: str,
     smtpd_server_port: int,
     message: email.message.Message,
-    received_messages: List[email.message.EmailMessage],
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
     auth_username: str,
     auth_password: str,
     client_tls_context: ssl.SSLContext,
@@ -162,14 +160,14 @@ async def test_send_with_login(
 async def test_send_via_socket(
     hostname: str,
     smtpd_server_port: int,
-    message: email.message.Message,
-    received_messages: List[email.message.EmailMessage],
+    mime_message: email.message.EmailMessage,
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
         sock.connect((hostname, smtpd_server_port))
 
-        errors, response = await send(
-            message,
+        errors, _ = await send(
+            mime_message,
             hostname=None,
             port=None,
             sock=sock,
@@ -179,17 +177,39 @@ async def test_send_via_socket(
         assert not errors
         assert len(received_messages) == 1
 
-        assert sock.fileno() > 0, "Socket unexpectedly closed"
+
+@pytest.mark.smtpd_options(tls=True)
+async def test_send_via_socket_tls_and_hostname(
+    hostname: str,
+    client_tls_context: ssl.SSLContext,
+    smtpd_server_port: int,
+    mime_message: email.message.EmailMessage,
+    received_messages: list[email.message.EmailMessage],
+) -> None:
+    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
+        sock.connect((hostname, smtpd_server_port))
+
+        errors, _ = await send(
+            mime_message,
+            hostname=hostname,
+            port=None,
+            sock=sock,
+            tls_context=client_tls_context,
+            use_tls=True,
+        )
+
+        assert not errors
+        assert len(received_messages) == 1
 
 
 async def test_send_via_socket_path(
     smtpd_server_socket_path: asyncio.AbstractServer,
     socket_path: Union[pathlib.Path, str, bytes],
-    message: email.message.Message,
-    received_messages: List[email.message.EmailMessage],
+    mime_message: email.message.EmailMessage,
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
-    errors, response = await send(
-        message,
+    errors, _ = await send(
+        mime_message,
         hostname=None,
         port=None,
         socket_path=socket_path,
@@ -200,13 +220,35 @@ async def test_send_via_socket_path(
     assert len(received_messages) == 1
 
 
+@pytest.mark.smtpd_options(tls=True)
+async def test_send_via_socket_path_with_tls(
+    smtpd_server_socket_path: asyncio.AbstractServer,
+    socket_path: Union[pathlib.Path, str, bytes],
+    hostname: str,
+    client_tls_context: ssl.SSLContext,
+    mime_message: email.message.EmailMessage,
+    received_messages: list[email.message.EmailMessage],
+) -> None:
+    errors, _ = await send(
+        mime_message,
+        hostname=hostname,
+        port=None,
+        socket_path=socket_path,
+        use_tls=True,
+        tls_context=client_tls_context,
+    )
+
+    assert not errors
+    assert len(received_messages) == 1
+
+
 async def test_send_with_mail_options(
     hostname: str,
     smtpd_server_port: int,
     recipient_str: str,
     sender_str: str,
     message_str: str,
-    received_messages: List[email.message.EmailMessage],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     errors, _ = await send(
         message_str,
@@ -228,7 +270,7 @@ async def test_send_with_rcpt_options(
     recipient_str: str,
     sender_str: str,
     message_str: str,
-    received_messages: List[email.message.EmailMessage],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     errors, _ = await send(
         message_str,
