@@ -2,15 +2,13 @@
 SMTP.sendmail and SMTP.send_message method testing.
 """
 
-import asyncio
 import copy
 import email.generator
 import email.header
 import email.message
-from typing import Any, Callable, Coroutine, List, Tuple, Type
+from typing import Any, Optional
 
 import pytest
-from aiosmtpd.smtp import SMTP as SMTPD
 
 from aiosmtplib import (
     SMTP,
@@ -19,15 +17,16 @@ from aiosmtplib import (
     SMTPResponseException,
     SMTPStatus,
 )
-from aiosmtplib.email import formataddr
 
-
-pytestmark = pytest.mark.asyncio()
+from .smtpd import (
+    mock_response_done,
+    mock_response_error_disconnect,
+    mock_response_bad_command_sequence,
+)
 
 
 async def test_sendmail_simple_success(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     sender_str: str,
     recipient_str: str,
     message_str: str,
@@ -44,7 +43,6 @@ async def test_sendmail_simple_success(
 
 async def test_sendmail_binary_content(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     sender_str: str,
     recipient_str: str,
     message_str: str,
@@ -61,7 +59,6 @@ async def test_sendmail_binary_content(
 
 async def test_sendmail_with_recipients_string(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     sender_str: str,
     recipient_str: str,
     message_str: str,
@@ -77,7 +74,6 @@ async def test_sendmail_with_recipients_string(
 
 async def test_sendmail_with_mail_option(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     sender_str: str,
     recipient_str: str,
     message_str: str,
@@ -91,18 +87,13 @@ async def test_sendmail_with_mail_option(
         assert response != ""
 
 
+@pytest.mark.smtpd_mocks(smtp_EHLO=mock_response_done)
 async def test_sendmail_without_size_option(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    smtpd_class: Type[SMTPD],
-    smtpd_mock_response_done: Callable,
-    monkeypatch: pytest.MonkeyPatch,
     sender_str: str,
     recipient_str: str,
     message_str: str,
 ) -> None:
-    monkeypatch.setattr(smtpd_class, "smtp_EHLO", smtpd_mock_response_done)
-
     async with smtp_client:
         errors, response = await smtp_client.sendmail(
             sender_str, [recipient_str], message_str
@@ -114,7 +105,6 @@ async def test_sendmail_without_size_option(
 
 async def test_sendmail_with_invalid_mail_option(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     sender_str: str,
     recipient_str: str,
     message_str: str,
@@ -133,7 +123,6 @@ async def test_sendmail_with_invalid_mail_option(
 
 async def test_sendmail_with_rcpt_option(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     sender_str: str,
     recipient_str: str,
     message_str: str,
@@ -155,27 +144,31 @@ async def test_sendmail_with_rcpt_option(
         )
 
 
-async def test_sendmail_simple_failure(
-    smtp_client: SMTP, smtpd_server: asyncio.AbstractServer
-) -> None:
+async def test_sendmail_simple_failure(smtp_client: SMTP) -> None:
     async with smtp_client:
         with pytest.raises(SMTPRecipientsRefused):
             #  @@ is an invalid recipient.
             await smtp_client.sendmail("test@example.com", ["@@"], "blah")
 
 
+async def test_sendmail_smtputf8_not_supported(smtp_client: SMTP) -> None:
+    async with smtp_client:
+        with pytest.raises(SMTPNotSupported, match="SMTPUTF8 is not supported"):
+            await smtp_client.sendmail(
+                "test@example.com",
+                ["børk@example.com"],
+                "blah",
+                mail_options=["SMTPUTF8"],
+            )
+
+
+@pytest.mark.smtpd_mocks(smtp_DATA=mock_response_error_disconnect)
 async def test_sendmail_error_silent_rset_handles_disconnect(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    smtpd_class: Type[SMTPD],
-    smtpd_mock_response_error_disconnect: Callable,
-    monkeypatch: pytest.MonkeyPatch,
     sender_str: str,
     recipient_str: str,
     message_str: str,
 ) -> None:
-    monkeypatch.setattr(smtpd_class, "smtp_DATA", smtpd_mock_response_error_disconnect)
-
     async with smtp_client:
         with pytest.raises(SMTPResponseException):
             await smtp_client.sendmail(sender_str, [recipient_str], message_str)
@@ -183,8 +176,7 @@ async def test_sendmail_error_silent_rset_handles_disconnect(
 
 async def test_rset_after_sendmail_error_response_to_mail(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
 ) -> None:
     """
     If an error response is given to the MAIL command in the sendmail method,
@@ -203,8 +195,7 @@ async def test_rset_after_sendmail_error_response_to_mail(
 
 async def test_rset_after_sendmail_error_response_to_rcpt(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
 ) -> None:
     """
     If an error response is given to the RCPT command in the sendmail method,
@@ -223,24 +214,18 @@ async def test_rset_after_sendmail_error_response_to_rcpt(
         assert received_commands[-1][0] == "RSET"
 
 
+@pytest.mark.smtpd_mocks(smtp_DATA=mock_response_bad_command_sequence)
 async def test_rset_after_sendmail_error_response_to_data(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    smtpd_class: Type[SMTPD],
-    monkeypatch: pytest.MonkeyPatch,
-    error_code: int,
     sender_str: str,
     recipient_str: str,
     message_str: str,
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
-    smtpd_mock_response_error_with_code: Callable[[SMTPD], Coroutine[Any, Any, None]],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
 ) -> None:
     """
     If an error response is given to the DATA command in the sendmail method,
     test that we reset the server session.
     """
-    monkeypatch.setattr(smtpd_class, "smtp_DATA", smtpd_mock_response_error_with_code)
-
     async with smtp_client:
         response = await smtp_client.ehlo()
         assert response.code == SMTPStatus.completed
@@ -248,15 +233,11 @@ async def test_rset_after_sendmail_error_response_to_data(
         with pytest.raises(SMTPResponseException) as excinfo:
             await smtp_client.sendmail(sender_str, [recipient_str], message_str)
 
-        assert excinfo.value.code == error_code
+        assert excinfo.value.code == SMTPStatus.bad_command_sequence
         assert received_commands[-1][0] == "RSET"
 
 
-async def test_send_message(
-    smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    message: email.message.Message,
-) -> None:
+async def test_send_message(smtp_client: SMTP, message: email.message.Message) -> None:
     async with smtp_client:
         errors, response = await smtp_client.send_message(message)
 
@@ -267,9 +248,8 @@ async def test_send_message(
 
 async def test_send_message_with_sender_and_recipient_args(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     message: email.message.Message,
-    received_messages: List[email.message.EmailMessage],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     sender = "sender2@example.com"
     recipients = ["recipient1@example.com", "recipient2@example.com"]
@@ -289,11 +269,10 @@ async def test_send_message_with_sender_and_recipient_args(
 
 async def test_send_message_with_cc_recipients(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     recipient_str: str,
     message: email.message.Message,
-    received_messages: List[email.message.EmailMessage],
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
 ) -> None:
     cc_recipients = ["recipient1@example.com", "recipient2@example.com"]
     message["Cc"] = ", ".join(cc_recipients)
@@ -306,7 +285,7 @@ async def test_send_message_with_cc_recipients(
     assert len(received_messages) == 1
     assert (
         received_messages[0]["X-RcptTo"]
-        == f'{recipient_str}, {", ".join(cc_recipients)}'
+        == f"{recipient_str}, {', '.join(cc_recipients)}"
     )
 
     assert received_commands[2][0] == "RCPT"
@@ -319,11 +298,10 @@ async def test_send_message_with_cc_recipients(
 
 async def test_send_message_with_bcc_recipients(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     recipient_str: str,
     message: email.message.Message,
-    received_messages: List[email.message.EmailMessage],
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
 ) -> None:
     bcc_recipients = ["recipient1@example.com", "recipient2@example.com"]
     message["Bcc"] = ", ".join(bcc_recipients)
@@ -345,11 +323,10 @@ async def test_send_message_with_bcc_recipients(
 
 async def test_send_message_with_cc_and_bcc_recipients(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     recipient_str: str,
     message: email.message.Message,
-    received_messages: List[email.message.EmailMessage],
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
 ) -> None:
     cc_recipient = "recipient2@example.com"
     message["Cc"] = cc_recipient
@@ -377,9 +354,8 @@ async def test_send_message_with_cc_and_bcc_recipients(
 
 async def test_send_message_recipient_str(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     message: email.message.Message,
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
 ) -> None:
     recipient_str = "1234@example.org"
     async with smtp_client:
@@ -395,7 +371,6 @@ async def test_send_message_recipient_str(
 
 async def test_send_message_mail_options(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     message: email.message.Message,
 ) -> None:
     async with smtp_client:
@@ -409,9 +384,7 @@ async def test_send_message_mail_options(
 
 
 async def test_send_multiple_messages_in_sequence(
-    smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    message: email.message.Message,
+    smtp_client: SMTP, message: email.message.Message
 ) -> None:
     message1 = copy.copy(message)
 
@@ -434,9 +407,7 @@ async def test_send_multiple_messages_in_sequence(
 
 
 async def test_send_message_without_recipients(
-    smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    message: email.message.Message,
+    smtp_client: SMTP, message: email.message.Message
 ) -> None:
     del message["To"]
 
@@ -446,9 +417,7 @@ async def test_send_message_without_recipients(
 
 
 async def test_send_message_without_sender(
-    smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    message: email.message.Message,
+    smtp_client: SMTP, message: email.message.Message
 ) -> None:
     del message["From"]
 
@@ -457,18 +426,18 @@ async def test_send_message_without_sender(
             await smtp_client.send_message(message)
 
 
+@pytest.mark.smtpd_options(smtputf8=True)
 async def test_send_message_smtputf8_sender(
-    smtp_client_smtputf8: SMTP,
-    smtpd_server_smtputf8: asyncio.AbstractServer,
+    smtp_client: SMTP,
     message: email.message.Message,
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
-    received_messages: List[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     del message["From"]
     message["From"] = "séndër@exåmple.com"
 
-    async with smtp_client_smtputf8:
-        errors, response = await smtp_client_smtputf8.send_message(message)
+    async with smtp_client:
+        errors, response = await smtp_client.send_message(message)
 
     assert not errors
     assert response != ""
@@ -483,17 +452,25 @@ async def test_send_message_smtputf8_sender(
     assert received_messages[0]["X-MailFrom"] == message["From"]
 
 
+@pytest.mark.smtpd_options(smtputf8=True)
+@pytest.mark.parametrize(
+    "mail_options",
+    (None, ["SMTPUTF8"]),
+    ids=("no_mail_options", "smtputf8_option"),
+)
 async def test_send_mime_message_smtputf8_recipient(
-    smtp_client_smtputf8: SMTP,
-    smtpd_server_smtputf8: asyncio.AbstractServer,
+    smtp_client: SMTP,
     mime_message: email.message.EmailMessage,
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
-    received_messages: List[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
+    mail_options: Optional[list[str]],
 ) -> None:
     mime_message["To"] = "reçipïént@exåmple.com"
 
-    async with smtp_client_smtputf8:
-        errors, response = await smtp_client_smtputf8.send_message(mime_message)
+    async with smtp_client:
+        errors, response = await smtp_client.send_message(
+            mime_message, mail_options=mail_options
+        )
 
     assert not errors
     assert response != ""
@@ -505,18 +482,18 @@ async def test_send_mime_message_smtputf8_recipient(
     assert received_messages[0]["X-RcptTo"] == ", ".join(mime_message.get_all("To"))
 
 
+@pytest.mark.smtpd_options(smtputf8=True)
 async def test_send_compat32_message_smtputf8_recipient(
-    smtp_client_smtputf8: SMTP,
-    smtpd_server_smtputf8: asyncio.AbstractServer,
+    smtp_client: SMTP,
     compat32_message: email.message.Message,
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
-    received_messages: List[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     recipient_bytes = bytes("reçipïént@exåmple.com", "utf-8")
     compat32_message["To"] = email.header.Header(recipient_bytes, "utf-8")
 
-    async with smtp_client_smtputf8:
-        errors, response = await smtp_client_smtputf8.send_message(compat32_message)
+    async with smtp_client:
+        errors, response = await smtp_client.send_message(compat32_message)
 
     assert not errors
     assert response != ""
@@ -531,10 +508,9 @@ async def test_send_compat32_message_smtputf8_recipient(
     )
 
 
+@pytest.mark.smtpd_options(smtputf8=False)
 async def test_send_message_smtputf8_not_supported(
-    smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    message: email.message.Message,
+    smtp_client: SMTP, message: email.message.Message
 ) -> None:
     message["To"] = "reçipïént2@exåmple.com"
 
@@ -543,26 +519,12 @@ async def test_send_message_smtputf8_not_supported(
             await smtp_client.send_message(message)
 
 
-async def test_send_message_with_formataddr(
-    smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    message: email.message.Message,
-) -> None:
-    message["To"] = formataddr(("æøå", "someotheruser@example.com"))
-
-    async with smtp_client:
-        errors, response = await smtp_client.send_message(message)
-
-    assert not errors
-    assert response != ""
-
-
+@pytest.mark.smtpd_options(smtputf8=False)
 async def test_send_compat32_message_utf8_text_without_smtputf8(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     compat32_message: email.message.Message,
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
-    received_messages: List[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     compat32_message["To"] = email.header.Header(
         "reçipïént <recipient2@example.com>", "utf-8"
@@ -589,12 +551,12 @@ async def test_send_compat32_message_utf8_text_without_smtputf8(
     ]
 
 
+@pytest.mark.smtpd_options(smtputf8=False)
 async def test_send_mime_message_utf8_text_without_smtputf8(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
     mime_message: email.message.EmailMessage,
-    received_commands: List[Tuple[str, Tuple[Any, ...]]],
-    received_messages: List[email.message.EmailMessage],
+    received_commands: list[tuple[str, tuple[Any, ...]]],
+    received_messages: list[email.message.EmailMessage],
 ) -> None:
     mime_message["To"] = "reçipïént <recipient2@example.com>"
 
@@ -619,11 +581,23 @@ async def test_send_mime_message_utf8_text_without_smtputf8(
     ]
 
 
-async def test_sendmail_empty_sender(
+@pytest.mark.smtpd_options(**{"smtputf8": False, "7bit": True})
+async def test_send_message_7bit(
     smtp_client: SMTP,
-    smtpd_server: asyncio.AbstractServer,
-    recipient_str: str,
-    message_str: str,
+    message: email.message.Message,
+    received_commands: list[tuple[str, tuple[Any, ...]]],
+) -> None:
+    async with smtp_client:
+        errors, response = await smtp_client.send_message(message)
+
+    assert not errors
+    assert response != ""
+
+    assert "BODY=8BITMIME" not in received_commands[1][1][1]
+
+
+async def test_sendmail_empty_sender(
+    smtp_client: SMTP, recipient_str: str, message_str: str
 ) -> None:
     async with smtp_client:
         errors, response = await smtp_client.sendmail("", [recipient_str], message_str)
